
# Todo: There's a scons-dev mailing list post which explains the need for this tool, but the scons-dev archive seems to be down at the moment. As soon as it's back, link to that post (entitled "Customizable in-archive paths for archive builders").

import os.path

import SCons.Builder
import SCons.Node.FS
import SCons.Util
import SCons.Errors
import SCons.Action

try:
    import zipfile
    def exists(env): return True
except ImportError:
    def exists(env): return False

def action(target, source, env):
    archive_path = env.get('archive_path', lambda x:x)
    def visit(arg, dirname, names):
        for name in names:
            path = os.path.join(dirname, name)
            if os.path.isfile(path):
                arg.write(path, str(archive_path(path)))
    zf = zipfile.ZipFile(str(target[0]), 'w', env.get('ZIPCOMPRESSION', zipfile.ZIP_DEFLATED))
    for s in source:
        if os.path.isdir(str(s)):
            os.path.walk(str(s), visit, zf)
        else:
            zf.write(str(s), str(archive_path(str(s))))
    zf.close()

action.strfunction = lambda target, source, env: 'Zipping ' + str(target[0])

builder_method_name = 'PythonZip'

def generate(env):
    """Add builder and construction variables to an Environment."""

    env['ZIPSUFFIX']  = '.zip'

    env['BUILDERS'][builder_method_name] = \
        SCons.Builder.Builder(action = action,
                              source_factory = SCons.Node.FS.default_fs.Entry,
                              suffix = '$ZIPSUFFIX',
                              multi = 1)
