
#include "BareModel.hpp"

#include "util/foreach.hpp"

namespace M3D {

BareModel::BareModel (GridSettings const & s)
  : settings_ (s)
{
  std::vector<FieldCoordinates> const v = s.fieldShape->generateFieldCoordinates(s.size);

  BOOST_FOREACH(FieldCoordinates const & c, v)
    if ((*s.gridShape)(c, s.size)) grid_[add_vertex(grid_)].coords = c;

  boost::graph_traits<Grid>::vertex_iterator b, e;
  boost::tie(b, e) = vertices(grid_);

  for (boost::graph_traits<Grid>::vertex_iterator i = b; i != e; ++i)
  for (boost::graph_traits<Grid>::vertex_iterator j = boost::next(i); j != e; ++j)
    if (s.fieldShape->adjacent(grid_[*i].coords, grid_[*j].coords, s.lessDetection))
      add_edge(*i, *j, grid_);
  // Efficiency: As you can see this is currently a O(n^2) operation. I'm sure we can do better than that.
}

} // namespace M3D
