
#ifndef M3D_CANVAS_HPP
#define M3D_CANVAS_HPP

#include <gtkglmm.h>

#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/optional.hpp>
#include <boost/noncopyable.hpp>

#include "FieldDrawers.hpp"
#include "Settings.hpp"
#include "Rotators.hpp"
#include "Observers.hpp"

#include <utility> // pair
#include <vector>

namespace M3D {

struct HistoricModel;

class Canvas : public Gtk::GL::DrawingArea, boost::noncopyable
{
  boost::shared_ptr<HistoricModel> model;
  CanvasSettings settings;
  boost::scoped_ptr<Rotator> rotator;
  boost::optional<Field> pointedToField;
    // If highlight style != no_highlighting, indicates the field that the cursor points to, if any. Note that it would be simpler and functionally harmless to maintain this variable regardless of the highlight style, but determining pointedToField is expensive so now people with slower systems can disable highlighting to increase performance.
  bool colorMapInBuffer, displayListOutdated, fieldSortingOutdated;
  boost::scoped_ptr<FieldDrawer>
    highlighted_neutral_drawer,
    highlighted_flag_drawer,
    highlighted_potentialMine_drawer;
      // Initialization for these is delayed until on_realize, because before that we cannot create textures.
  RGBAcolor backgroundColor;
  GLuint displayList;
  typedef std::pair<double, double> MouseLocation;
  boost::optional<MouseLocation> prevMousePosition;
    // Needed for mouse handling. Only used in onMouseEvent(). Optional because there is none at the first invokation of onMouse() yet.
  boost::optional<MouseLocation> dragStartPoint;
    // these are only set when the cursor is still inside the drag box
  std::vector<FieldDrawer> fieldDrawers;
  GLfloat cameraDistance /* from center of model */, sphereRadius;
  typedef std::vector<std::pair<float /* distance to eye */, Field> > FieldSorting;
  FieldSorting fieldSorting;

  void draw (FieldVisualization);
  void paintColorMap ();

  bool highlighted (Field) const;
  void setProjectionMatrix () const;
  void loadSkin ();
  RGBcolor encodeField (Field) const;
  Field decodeField (RGBcolor const &) const;
  boost::optional<Field> getFieldUnder (double x, double y);

  bool on_button_press_event (GdkEventButton *);
  bool on_button_release_event (GdkEventButton *);
  bool on_motion_notify_event (GdkEventMotion *);
  void on_realize ();
  bool on_configure_event (GdkEventConfigure *);
  bool on_expose_event (GdkEventExpose *);
  void onDrag (double,double,double,double);
  void onClick (double x, double y, bool primary);
  void onSphereRadiusChange ();
  void onFieldClick (Field, bool primary);
  void onOperation (ObservedModel &, Operation const &);

  public:

    void invalidate (); // "refresh"
    void set (boost::shared_ptr<HistoricModel>);
    void set (CanvasSettings const &);
    explicit Canvas (boost::shared_ptr<HistoricModel>, CanvasSettings const &);
    ~Canvas ();
};

} // namespace M3D

#endif // header guard
