
#include "Observers.hpp"
#include "Operations.hpp"

namespace
{
  template <typename T>
  T positive_modulo (T const l, T const r) { return ((l % r) + r) % r; }
}

namespace M3D {

void FlagObserver::onOperation (ObservedModel & model, Operation const & op)
{
  if (FlagOperation const * const p = dynamic_cast<FlagOperation const *>(&op))
    flaggedProperty[p->field] = p->flagLevel;
  else if (dynamic_cast<RestartOperation const *>(&op))
    std::fill(flaggedProperty.begin(), flaggedProperty.end(), none);
}

void FlagObserver::cycleFlagLevel (Field const f, ObservedModel & model)
{
  if (playState(model) != playing || opened(model)[f]) return;

  FlagOperation o;
  o.field = f;
  o.flagLevel = FlagLevel(positive_modulo(flagLevel(model)[f] + 1, int(flagLevel_count)));
  if (!cycleThroughRisky && o.flagLevel == risky) o.flagLevel = FlagLevel(int(o.flagLevel) + 1);
  model.apply(o);
}

FlaggedProperty const & flagLevel (ObservedModel const & model)
{
  return model.get<FlagObserver>().flaggedProperty;
}

void cycleFlagLevel (Field const f, ObservedModel & model)
{
  model.get<FlagObserver>().cycleFlagLevel(f, model);
}

void flag (Field const f, ObservedModel & model)
{
  if (playState(model) != playing ||
      flagLevel(model)[f] == flagged ||
      opened(model)[f]) return;

  FlagOperation o;
  o.field = f;
  o.flagLevel = flagged;
  model.apply(o);
}

} // namespace M3D
