
#ifndef M3D_GRID_HPP
#define M3D_GRID_HPP

#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/properties.hpp>

#include "geometry.hpp"

#include "util/FixedVector.hpp"

namespace boost
{
  template <std::size_t MaxSize>
  struct fixed_vecS {};

  template <typename ValueType, std::size_t MaxSize>
  struct container_gen<fixed_vecS<MaxSize>, ValueType>
  {
    typedef FixedVector<ValueType, MaxSize> type;
  };

  template <std::size_t MaxSize>
  struct parallel_edge_traits<fixed_vecS<MaxSize> >
  {
    typedef allow_parallel_edge_tag type;
  };

  namespace graph_detail // Todo: There must be a better way.
  {
    template <typename T, std::size_t MaxSize>
    vector_tag container_category (FixedVector<T, MaxSize> const &)
    { return vector_tag(); }
  }
}

namespace M3D {

struct FieldProperties { FieldCoordinates coords; };

std::size_t const greatest_adjacency_count = 32;
  // Sync: No field shape must have more adjacencies for any single field.

typedef
 boost::adjacency_list<
  boost::fixed_vecS<greatest_adjacency_count>, // edge storage (for each vertex)
  boost::vecS, // vertex storage
  boost::undirectedS,
  FieldProperties
 >
 Grid;

typedef boost::graph_traits<Grid>::vertex_descriptor Field;

} // namespace M3D

#endif // header guard
