
#ifndef M3D_HISTORICMODEL_HPP
#define M3D_HISTORICMODEL_HPP

#include <stack>

#include "ObservedModel.hpp"
#include "Operations.hpp"

namespace M3D {

// Undo/redo cannot be implemented as ordinary operations because they operate at an operation "meta" level, affecting how, if, and when operations themselves are applied.

// "Historic" is used not to indicate that instances are from the past, but that instances are aware of their place in history. (Previously this class was called "HistoryAwareModel", which was more accurate but simply too long.)

class HistoricModel : public ObservedModel
{
  typedef ObservedModel BaseModel;
  typedef std::stack<CloningPtr<Operation> > OperationGroup;

  std::stack<OperationGroup> past, future;
  bool recording;

  void onOperation (Operation const &);

  public:

    virtual void apply (Operation const &);

    HistoricModel (BaseModel const &);

    void setHistoryMark ();
    void historyStep (bool forward); // will have no effect if !canHistoryStep(forward)
    bool canHistoryStep (bool forward) const;
    void clearPast ();
};

inline void undo (HistoricModel & m) { m.historyStep(false); }
inline void redo (HistoricModel & m) { m.historyStep(true); }

inline bool canUndo (HistoricModel & m) { return m.canHistoryStep(false); }
inline bool canRedo (HistoricModel & m) { return m.canHistoryStep(true); }

} // namespace M3D

#endif // header guard
