
#include "ObservedModel.hpp"
#include "Operations.hpp"

#include <boost/bind.hpp>
#include <boost/lambda/lambda.hpp>

#include "util/ScopeGuard.hpp"

namespace M3D {

void ObservedModel::apply (Operation const & op)
{
  if (std::find_if(pending.begin(), pending.end(),
                   boost::bind(&Operation::equal, _1, boost::ref(op))) != pending.end())
    return;

  pending.push_back(CloningPtr<Operation>(op.clone()));

  if (applying) return;

  applying = true;

  ScopeGuard const sg (boost::lambda::var(applying) = false);

  typedef boost::multi_index::index<Observers, order>::type OrderIndex;
  OrderIndex const & order_index = boost::multi_index::get<order>(observers);

  while (!pending.empty())
  {
    BareModel::apply(*pending.front());

    CloningPtr<Operation> const o = pending.front();
    pending.pop_front();
      // It's important that we pop _before_ we allows observers to observe this operation, because otherwise observers cannot submit operations identical to the one currently being observed. (For example, AutoStarter does this when it's examining a RestartOperation and decides that another RestartOperation needs to be submitted.)

    for (OrderIndex::iterator i = order_index.begin(), e = order_index.end(); i != e; ++i)
      i->instance->onOperation(*this, *o);
  }
}

ObservedModel::ObservedModel (BareModel const & model) : BareModel (model), applying (false) {}

ObservedModel::Observer * ObservedModel::getObserver (TypeConstant const c) const
{
  Observers::const_iterator const i = observers.find(c);
  return (i == observers.end() ? 0 : i->instance.get());
}

void ObservedModel::remove (TypeConstant const c)
{
  observers.erase(c);
}

} // namespace M3D
