
#ifndef M3D_ROTATORS_HPP
#define M3D_ROTATORS_HPP

#include <boost/noncopyable.hpp>
#include <boost/math/quaternion.hpp>
#include "util/VirtuallyDestructed.hpp"

namespace M3D {

struct Rotator : boost::noncopyable, VirtuallyDestructed
{
  virtual boost::math::quaternion<float> getRotation () const = 0;

  virtual void onDrag (float xfrom, float xto, float yfrom, float yto) = 0;
    // Called by Canvas. All parameters must be in the range [-0.5,0.5].
};

struct RotatorFactory : boost::noncopyable, protected VirtuallyDestructed
  // RotatorFactory is only VirtuallyDestructed to suppress warnings, it is not actually deleted virtually anywhere.
{
  virtual Rotator * create (float speed, Rotator const * prev = 0) const = 0;
    // Returns a new Rotator allocated with "new". Todo: consider using a smart pointer return type.
};

// A rotator's speed can not be changed.

} // namespace M3D

#endif // header guard
