
#ifndef UTIL_CLONINGPTR_HPP
#define UTIL_CLONINGPTR_HPP

// Extdep: There's a good chance Boost 1.34 will have something very much like this, or facilities that make this easier to implement (for example a policy based smart pointer).

#include <algorithm> // swap

template <typename T>
class CloningPtr
// Can be 0. Copies using T's clone() member, which must return a new object allocated with 'new'. Deletes on destruction.
{
  T * p;

  public:

    explicit CloningPtr (T * const p_ = 0) : p (p_) {}
    template <typename U> CloningPtr (CloningPtr<U> const & other) : p (other.p ? other.p->clone() : 0) {}
    CloningPtr (CloningPtr const & other) : p (other.p ? other.p->clone() : 0) {}
    CloningPtr & operator= (CloningPtr t) { swap(t); return *this; }
    ~CloningPtr () { delete p; }

    void swap (CloningPtr & other) { std::swap(p, other.p); }
    void reset (T * const p_ = 0) { delete p; p = p_; }

    T * operator-> () const { assert(p); return p; }
    T & operator* () const { assert(p); return *p; }
    T * get () const { return p; }
};

template <typename T> T * get_pointer (CloningPtr<T> const & cp) { return cp.get(); } // used by Boost.Bind

#endif // header guard
