
#include "bfstream.hpp"
#include <exception>
#include <cerrno>
#include <cstring> // strerror
#include <boost/format.hpp>
#include "gettext.hpp"

namespace
{
  std::string error_string (std::ios_base::openmode const m)
  {
    bool const in = m & std::ios_base::in, out = m & std::ios_base::out;
    assert(in || out);
    return (in && out ? _("Could not open \"%1%\" for reading and writing because: %2%.")
                      : (in ? _("Could not open \"%1%\" for reading because: %2%.")
                            : _("Could not open \"%1%\" for writing because: %2%.")));
      // This seems awfully redundant, but if we compose these strings by bits and pieces translation to other languages with different grammatical structure will likely become impossible.
  }
}

file_open_error::file_open_error (boost::filesystem::path const & path, std::ios_base::openmode const mode, int const errcode)
  : path_ (path), mode_ (mode), errcode_ (errcode),
    what_ (str(boost::format(error_string(mode)) % path.native_file_string() % std::strerror(errcode)))
{}

bfstream::bfstream (boost::filesystem::path const & p, std::ios_base::openmode const mode)
{
  open(p, mode);
  if (!good())
  {
    assert(errno);
    throw file_open_error(p, mode, errno);
  }

  exceptions(std::ios_base::badbit | std::ios_base::failbit);
    // Todo: Is throwing on failbit appropriate? Look into it.
}

bifstream::bifstream (boost::filesystem::path const & p) : bfstream (p, std::ios_base::in) {}
bofstream::bofstream (boost::filesystem::path const & p) : bfstream (p, std::ios_base::out) {}
