
// Whether a member was serialized directly or symbolically used to be determined only by type. Now, it can be specified for individual members, so that even members of common types such as std::string can be serialized symbolically if appropriate.

#ifndef UTIL_SERIALIZATION_SYMBOLS_HPP
#define UTIL_SERIALIZATION_SYMBOLS_HPP

#include <boost/multi_index/member.hpp>
#include <boost/multi_index/ordered_index.hpp>
#include <boost/multi_index_container.hpp>

#include <boost/utility/enable_if.hpp>
#include <boost/type_traits/is_enum.hpp>

#include "OverloadTag.hpp"

#include <string>
#include <utility>
#include <stdexcept>

template <typename T>
struct serialization_symbols // todo: just rename to bimap or something (for "bidirectional map")
{
  typedef std::pair<T, std::string> P;

  typedef boost::multi_index_container<
    P,
    boost::multi_index::indexed_by<
     boost::multi_index::ordered_unique<boost::multi_index::member<P, T, &P::first> >,
     boost::multi_index::ordered_unique<boost::multi_index::member<P, std::string, &P::second> >
    >
   > type; // this is simply a bidirectional map
};

template <typename T, typename Pod, T (Pod:: * M)>
struct SerializationSymbolOverloadTag : OverloadTag_base {};

template <typename T, typename Pod, T (Pod:: * M)>
typename serialization_symbols<T>::type const & get_serialization_symbols ()
{
  static typename serialization_symbols<T>::type const r = make_serialization_symbols(SerializationSymbolOverloadTag<T, Pod, M>());
  return r;
}

char make_serialization_symbols (OverloadTag_base);

template <typename T, typename Pod, T (Pod:: * M)>
struct is_symbolically_serialized
{
  enum { value = sizeof(make_serialization_symbols(SerializationSymbolOverloadTag<T, Pod, M>())) != 1 };
};

template <typename T, typename Pod, T (Pod:: * M)>
std::string const & to_symbol (T const & t)
{
  typedef typename serialization_symbols<T>::type::template nth_index<0>::type Index;
  Index const & index = get_serialization_symbols<T, Pod, M>().template get<0>();
  typename Index::const_iterator const i = index.find(t);
  if (i == index.end()) throw std::runtime_error("symbol lookup failed");
  return i->second;
}

template <typename T, typename Pod, T (Pod:: * M)>
T from_symbol (std::string const & s)
{
  typedef typename serialization_symbols<T>::type::template nth_index<1>::type Index;
  Index const & index = get_serialization_symbols<T, Pod, M>().template get<1>();
  typename Index::const_iterator const i = index.find(s);
  if (i == index.end()) throw std::runtime_error("symbol lookup failed: " + s);
  return i->first;
}

#endif // header guard
